/*
 * compile command for Linux
 * g++ -o bin/StarIO_Sample -W main.cpp -lStarIOPort -lstarusb -Wl,--rpath,'$ORIGIN'
 *
 * compile command for Mac OS X
 * g++ -I./include -L./bin -o bin/StarIO_Sample -W main.cpp -lStarIOPort
 */

#include <starmicronics/StarIOPort.h>


// forward declarations
void WritePortUsage();
void GetParsedStatusUsage();
void ResetDeviceUsage();
void GetOnlineStatusUsage();
void CheckedBlockUsage();
void ReadPortUsage();



#if defined(WIN32) || defined(_WIN32_WCE)

#include <wchar.h>

// unicode
#define CONSOLE_OUT wprintf
#define CONSOLE_IN wscanf
#define CONSOLE_IN_CHAR getwchar

#define STR_CMP wcscmp
#define STR_TO_INT _wtoi

#define STR_ELEM wchar_t
#define STR(x) L ## x

wchar_t g_portName[100 + 1] = {0};
wchar_t g_portSettings[100 + 1] = {0};
const UINT32 g_portTimeout = 10 * 1000;

int wmain(int argc, wchar_t *argv[ ], wchar_t *envp[ ])

#else

#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <unistd.h>

// ansi
#define CONSOLE_OUT printf
#define CONSOLE_IN scanf
#define CONSOLE_IN_CHAR getchar

#define STR_CMP strcmp
#define STR_TO_INT atoi

#define STR_ELEM char
#define STR(x) x

char g_portName[100 + 1] = {0};
char g_portSettings[100 + 1] = {0};
const UINT32 g_portTimeout = 10 * 1000;

int main(int argc, char * argv[])

#endif
{
    CONSOLE_OUT(STR("Enter the port name: "));
    CONSOLE_IN(STR("%[^\t\n]"), g_portName); CONSOLE_IN_CHAR();

    CONSOLE_OUT(STR("Enter the port settings: "));
    CONSOLE_IN(STR("%[^\t\n]"), g_portSettings); CONSOLE_IN_CHAR();

    while(true)
    {
        CONSOLE_OUT(STR("[1] Write Port\n"));
        CONSOLE_OUT(STR("[2] Get Parsed Status\n"));
        CONSOLE_OUT(STR("[3] Reset Device\n"));
        CONSOLE_OUT(STR("[4] Get Online Status\n"));
        CONSOLE_OUT(STR("[5] Checked Block\n"));
        CONSOLE_OUT(STR("[6] Read Port\n"));
        CONSOLE_OUT(STR("[x] Quit\n"));

        STR_ELEM answerString[10];

        CONSOLE_OUT(STR("Select one of the above options: "));
        CONSOLE_IN(STR("%s"), answerString); CONSOLE_IN_CHAR();

        if(STR_CMP(answerString, STR("x")) == 0)
        {
            return 0;
        }

        int answer = STR_TO_INT(answerString);

        switch(answer)
        {
        case 1:
            WritePortUsage();
            break;
        case 2:
            GetParsedStatusUsage();
            break;
        case 3:
            ResetDeviceUsage();
            break;
        case 4:
            GetOnlineStatusUsage();
            break;
        case 5:
            CheckedBlockUsage();
            break;
        case 6:
            ReadPortUsage();
            break;
        default:
            CONSOLE_OUT(STR("Invalid Option\n"));
            break;
        }

        CONSOLE_OUT(STR("\n"));
    }

    return 0;
}

INT64 WritePortHelper(void * port, UCHAR const * writeBuffer, UINT32 writeBufferSize)
{
    UINT32 zeroProgressOccurances = 0;
    UINT32 totalSizeCommunicated = 0;
    while ((totalSizeCommunicated < writeBufferSize) && (zeroProgressOccurances < 2)) // adjust zeroProgressOccurances threshold as required
    {
        UINT32 sizeCommunicated = 0;
        if(WritePort(port, &writeBuffer[totalSizeCommunicated], writeBufferSize - totalSizeCommunicated, &sizeCommunicated) == 0)
        {
            return -1;
        }

        if (sizeCommunicated == 0)
        {
            zeroProgressOccurances++;
        }
        else
        {
            totalSizeCommunicated += sizeCommunicated;
            zeroProgressOccurances = 0;
        }
    }

    return (INT64)totalSizeCommunicated;
}

void WritePortUsage()
{
    void * port = OpenPort(g_portName, g_portSettings, g_portTimeout);
    if(port == NULL)
    {
        CONSOLE_OUT(STR("Failure - OpenPort\n"));
        return;
    }

    UCHAR const * command = (UCHAR const *)"Hello World!\n";
    UINT32 const commandSize = sizeof("Hello World!\n") - 1;

    INT64 totalSizeCommunicated = WritePortHelper(port, command, commandSize);
    if (totalSizeCommunicated == -1)
    {
        CONSOLE_OUT(STR("Failure - WritePort.\n"));
        ClosePort(port);
        return;
    }
    else if (totalSizeCommunicated != (INT64)commandSize)
    {
        CONSOLE_OUT(STR("Error - not all data could be written out.\n"));
        ClosePort(port);
        return;
    }

    ClosePort(port);

    CONSOLE_OUT(STR("Success\n"));
}

void GetParsedStatusUsage()
{
    void * port = OpenPort(g_portName, g_portSettings, g_portTimeout);
    if (port == NULL)
    {
        CONSOLE_OUT(STR("Failure - OpenPort\n"));
        return;
    }

    StarPrinterStatus_2 starPrinterStatus;
    if (GetParsedStatusEx(port, &starPrinterStatus, 2) == 0)
    {
        CONSOLE_OUT(STR("Failure - GetParsedStatusEx\n"));
        ClosePort(port);
        return;
    }

    if (starPrinterStatus.offline == SM_TRUE)
    {
        if(starPrinterStatus.coverOpen == SM_TRUE)
        {
            CONSOLE_OUT(STR("The printer is offline - cover open.\n"));
        }
        else if(starPrinterStatus.receiptPaperEmpty == SM_TRUE)
        {
            CONSOLE_OUT(STR("The printer is offline - paper empty.\n"));
        }
        else
        {
            CONSOLE_OUT(STR("The printer is offline.\n"));
        }
    }
    else
    {
        CONSOLE_OUT(STR("The printer is online.\n"));
    }

    ClosePort(port);

    CONSOLE_OUT(STR("Success\n"));
}

void ResetDeviceUsage()
{
    void * port = OpenPort(g_portName, g_portSettings, g_portTimeout);
    if (port == NULL)
    {
        CONSOLE_OUT(STR("Failure - OpenPort\n"));
        return;
    }

    if (ResetDevice(port) == SM_FALSE)
    {
        CONSOLE_OUT(STR("Failure - ResetDevice\n"));
    }

    ClosePort(port);

    CONSOLE_OUT(STR("Success\n"));
}

void GetOnlineStatusUsage()
{
    void * port = OpenPort(g_portName, g_portSettings, g_portTimeout);
    if (port == NULL)
    {
        CONSOLE_OUT(STR("Failure - OpenPort\n"));
        return;
    }

    SM_BOOLEAN onlineStatus;
    if(GetOnlineStatus(port, &onlineStatus) == SM_FALSE)
    {
        CONSOLE_OUT(STR("Failure - GetOnlineStatus\n"));
        ClosePort(port);
        return;
    }

    if(onlineStatus == SM_TRUE)
    {
        CONSOLE_OUT(STR("The printer is online.\n"));
    }
    else
    {
        CONSOLE_OUT(STR("The printer is offline.\n"));
    }

    ClosePort(port);

    CONSOLE_OUT(STR("Success\n"));
}

void CheckedBlockUsage()
{
    void * port = OpenPort(g_portName, g_portSettings, g_portTimeout);
    if (port == NULL)
    {
        CONSOLE_OUT(STR("Failure - OpenPort\n"));
        return;
    }

    StarPrinterStatus_2 starPrinterStatus;

    if (BeginCheckedBlockEx(port, &starPrinterStatus, 2) == SM_FALSE)
    {
        CONSOLE_OUT(STR("Failure - BeginCheckedBlockEx\n"));
        ClosePort(port);
        return;
    }
    if(starPrinterStatus.offline == SM_TRUE)
    {
        CONSOLE_OUT(STR("Error - Cannot proceed with printing because the printer is offline.\n"));
        ClosePort(port);
        return;
    }

    UCHAR const * command = (UCHAR const *)"Hello World!\n";
    UINT32 const commandSize = sizeof("Hello World!\n") - 1;

    for (int count = 0; count < 50; count++)
    {
        INT64 totalSizeCommunicated = WritePortHelper(port, command, commandSize);
        if (totalSizeCommunicated == -1)
        {
            CONSOLE_OUT(STR("Failure - WritePort.\n"));
            ClosePort(port);
            return;
        }
        else if (totalSizeCommunicated != (INT64)commandSize)
        {
            CONSOLE_OUT(STR("Error - not all data could be written out.\n"));
            ClosePort(port);
            return;
        }
    }

    if (EndCheckedBlockEx(port, &starPrinterStatus, 2) == SM_FALSE)
    {
        CONSOLE_OUT(STR("Failure - EndCheckedBlockEx\n"));
        ClosePort(port);
        return;
    }
    if(starPrinterStatus.offline == SM_TRUE)
    {
        CONSOLE_OUT(STR("Error - The printer is offline indicating that the printing failed.\n"));
        ClosePort(port);
        return;
    }

    ClosePort(port);

    CONSOLE_OUT(STR("Success\n"));
}

void ReadPortUsage()
{
    void * port = OpenPort(g_portName, g_portSettings, g_portTimeout);
    if (port == NULL)
    {
        CONSOLE_OUT(STR("Failure - OpenPort\n"));
        return;
    }

    UCHAR firmwareNameRequestCommand[5] = { 0x1b, (UCHAR)'#', (UCHAR)'*', 0x0a, 0x00 };
    INT64 totalSizeCommunicated = WritePortHelper(port, firmwareNameRequestCommand, sizeof(firmwareNameRequestCommand));
    if (totalSizeCommunicated == -1)
    {
        CONSOLE_OUT(STR("Failure - WritePort.\n"));
        ClosePort(port);
        return;
    }
    else if (totalSizeCommunicated != (INT64)sizeof(firmwareNameRequestCommand))
    {
        CONSOLE_OUT(STR("Error - not all data could be written out.\n"));
        ClosePort(port);
        return;
    }

    STR_ELEM firmwareNameBuffer[100];
    int firmwareNameBufferIdx = 0;
    int tryCount = 0;

    int wrongCount = 0;
    const int readState_init = 0, readState_inFirmwareName_1 = 1, readState_inFirmwareName_2 = 2, readState_inFirmwareName_3 = 3, readState_inFirmwareName_4 = 4, readState_inFirmwareName_5 = 5, readState_done = 6;
    int readState = readState_init;

    while ((readState != readState_done) && (wrongCount < 50))
    {
        UCHAR inputBuffer[1024];
        UINT32 sizeCommunicated = 0;
        if (ReadPort(port, inputBuffer, sizeof(inputBuffer), &sizeCommunicated) == SM_FALSE)
        {
            CONSOLE_OUT(STR("Failure - ReadPort.\n"));
            ClosePort(port);
            return;
        }
        if (sizeCommunicated == 0)
        {
            tryCount++;
            if (tryCount > 2)
            {
                break;
            }
        }

        for (UINT32 i = 0; i < sizeCommunicated; i++)     // shin : evade warning (int  <->  unsigned int)
        {
            if ((readState == readState_init) && (inputBuffer[i] == 0x1b))
            {
                readState = readState_inFirmwareName_1;
            }
            else if ((readState == readState_inFirmwareName_1) && (inputBuffer[i] == (UCHAR)'#'))
            {
                readState = readState_inFirmwareName_2;
            }
            else if ((readState == readState_inFirmwareName_2) && (inputBuffer[i] == (UCHAR)'*'))
            {
                readState = readState_inFirmwareName_3;
            }
            else if ((readState == readState_inFirmwareName_3) && (inputBuffer[i] == (UCHAR)','))
            {
                readState = readState_inFirmwareName_4;
            }
            else if ((readState == readState_inFirmwareName_4) && (inputBuffer[i] == 0x0a))
            {
                readState = readState_inFirmwareName_5;
            }
            else if ((readState == readState_inFirmwareName_5) && (inputBuffer[i] == 0x00))
            {
                readState = readState_done;
                break;
            }
            else if (readState == readState_inFirmwareName_4) // must be last before the catch-all
            {
                firmwareNameBuffer[firmwareNameBufferIdx++] = (STR_ELEM)inputBuffer[i];
            }
            else
            {
                readState = readState_init;
                firmwareNameBufferIdx = 0;
                wrongCount++;
            }
        }
    }

    firmwareNameBuffer[firmwareNameBufferIdx] = 0;

    if (readState == readState_done)
    {
        CONSOLE_OUT(STR("Success - firmware name = %s\n"), firmwareNameBuffer);
    }
    else
    {
        CONSOLE_OUT(STR("Failure - could not read in the firmware name.\n"));
    }

    ClosePort(port);

    CONSOLE_OUT(STR("Success\n"));
}
