Imports System.Text

Imports StarMicronics.StarIO ' added as a reference from the "Dependencies" directory 
' requires StarIOPort.dll, which is copied to the output directory by the Post-Build event (32-bit / 64-bit platform specific)

Module Main

    Sub Main()
        Console.Write("Enter the port name: ")
        Dim portName As String = Console.ReadLine()

        Console.Write("Enter the port settings: ")
        Dim portSettings As String = Console.ReadLine()

        Dim run As Boolean = True
        While run = True
            Console.WriteLine()

            Console.WriteLine("[1] Write Port")
            Console.WriteLine("[2] Get Status")
            Console.WriteLine("[3] Reset Device")
            Console.WriteLine("[4] Online Status")
            Console.WriteLine("[5] Checked Block")
            Console.WriteLine("[6] Read Port")
            Console.WriteLine("[0] Exit Program")
            Console.Write("Select one of the above options: ")

            Dim selection As UInteger
            Try
                selection = UInteger.Parse(Console.ReadLine())
            Catch
                selection = 0
            End Try

            Select Case selection
                Case 1
                    WritePortUsage(portName, portSettings)
                    Exit Select
                Case 2
                    GetStatusUsage(portName, portSettings)
                    Exit Select
                Case 3
                    ResetDeviceUsage(portName, portSettings)
                    Exit Select
                Case 4
                    GetOnlineStatusUsage(portName, portSettings)
                    Exit Select
                Case 5
                    CheckedBlockUsage(portName, portSettings)
                    Exit Select
                Case 6
                    ReadPortUsage(portName, portSettings)
                    Exit Select
                Case 0
                    run = False
                    Exit Select
                Case Else
                    Console.WriteLine("Invalid option")
                    Exit Select
            End Select
        End While
    End Sub

    Public Sub WritePortUsage(ByVal portName As String, ByVal portSettings As String)
        Dim activePort As IPort = Nothing
        Try
            activePort = StarMicronics.StarIO.Factory.I.GetPort(portName, portSettings, 10 * 1000)

            Dim command As Byte() = ASCIIEncoding.ASCII.GetBytes("Hello World." & vbLf)

            Dim totalSizeCommunicated As UInteger = WritePortHelper(activePort, command) ' see below
            If totalSizeCommunicated <> command.Length Then
                Console.WriteLine("Error - not all data could be written out.")
                Exit Sub
            Else
                Console.WriteLine("Success")
            End If
        Catch e As PortException
            Console.WriteLine("Failure - see stack trace:" & vbLf & e.StackTrace)
        Finally
            If activePort IsNot Nothing Then
                StarMicronics.StarIO.Factory.I.ReleasePort(activePort)
            End If
        End Try
    End Sub

    Public Sub GetStatusUsage(ByVal portName As String, ByVal portSettings As String)
        Dim activePort As IPort = Nothing
        Try
            activePort = StarMicronics.StarIO.Factory.I.GetPort(portName, portSettings, 10 * 1000)

            Dim printerStatus As StarPrinterStatus = activePort.GetParsedStatus()
            If printerStatus.Offline = False Then
                Console.WriteLine("The printer is online")
            Else
                Console.WriteLine("The printer is offline")
                If printerStatus.ReceiptPaperEmpty = True Then
                    Console.WriteLine(vbTab & "Paper empty")
                End If

                If printerStatus.CoverOpen = True Then
                    Console.WriteLine(vbTab & "over open")
                End If

            End If

            Console.WriteLine("Success")
        Catch e As PortException
            Console.WriteLine("Failure - see stack trace:" & vbLf & e.StackTrace)
        Finally
            If activePort IsNot Nothing Then
                StarMicronics.StarIO.Factory.I.ReleasePort(activePort)
            End If
        End Try
    End Sub

    Public Sub ResetDeviceUsage(ByVal portName As String, ByVal portSettings As String)
        Dim activePort As IPort = Nothing
        Try
            activePort = StarMicronics.StarIO.Factory.I.GetPort(portName, portSettings, 10 * 1000)

            activePort.ResetDevice()

            Console.WriteLine("Success")
        Catch e As PortException
            Console.WriteLine("Failure - see stack trace:" & vbLf & e.StackTrace)
        Finally
            If activePort IsNot Nothing Then
                StarMicronics.StarIO.Factory.I.ReleasePort(activePort)
            End If
        End Try
    End Sub

    Public Sub GetOnlineStatusUsage(ByVal portName As String, ByVal portSettings As String)
        Dim activePort As IPort = Nothing
        Try
            activePort = StarMicronics.StarIO.Factory.I.GetPort(portName, portSettings, 10 * 1000)

            Dim onlineStatus As Boolean = activePort.GetOnlineStatus()
            If onlineStatus = True Then
                Console.WriteLine("The printer is online")
            Else
                Console.WriteLine("The Printer is offline")
            End If

            Console.WriteLine("Success")
        Catch e As PortException
            Console.WriteLine("Failure - see stack trace:" & vbLf & e.StackTrace)
        Finally
            If activePort IsNot Nothing Then
                StarMicronics.StarIO.Factory.I.ReleasePort(activePort)
            End If
        End Try
    End Sub

    Public Sub CheckedBlockUsage(ByVal portName As String, ByVal portSettings As String)
        Dim activePort As IPort = Nothing
        Try
            activePort = StarMicronics.StarIO.Factory.I.GetPort(portName, portSettings, 10 * 1000)

            Dim beginningPrinterStatus As StarPrinterStatus = activePort.BeginCheckedBlock()
            If beginningPrinterStatus.Offline = True Then
                Console.WriteLine("The printer is offline, so printing will not proceed.")
                Exit Sub
            End If

            Dim command As Byte() = ASCIIEncoding.ASCII.GetBytes("Hello World." & vbLf)
            For count As Integer = 0 To 49
                Dim totalSizeCommunicated As UInteger = WritePortHelper(activePort, command) ' see below
                If totalSizeCommunicated <> command.Length Then
                    Console.WriteLine("Error - not all data could be written out.")
                    Exit Sub
                End If
            Next

            Dim endingPrinterStatus As StarPrinterStatus = activePort.EndCheckedBlock()
            If endingPrinterStatus.Offline = True Then
                Console.WriteLine("The printer is offline, meaning that printing failed.")
            Else
                Console.WriteLine("Success")
            End If
        Catch e As PortException
            Console.WriteLine("Failure - see stack trace:" & vbLf & e.StackTrace)
        Finally
            If activePort IsNot Nothing Then
                StarMicronics.StarIO.Factory.I.ReleasePort(activePort)
            End If
        End Try
    End Sub

    Public Sub ReadPortUsage(ByVal portName As String, ByVal portSettings As String)
        Dim activePort As IPort = Nothing
        Try
            activePort = StarMicronics.StarIO.Factory.I.GetPort(portName, portSettings, 10 * 1000)

            ' send command to request firmware name 
            Dim firmwareNameRequestCommand As Byte() = New Byte() {&H1B, Asc("#"c), Asc("*"c), &HA, &H0}
            Dim totalSizeCommunicated As UInteger = WritePortHelper(activePort, firmwareNameRequestCommand) ' see below
            If totalSizeCommunicated <> firmwareNameRequestCommand.Length Then
                Console.WriteLine("Error - not all data could be written out.")
                Exit Sub
            End If

            Dim firmwareNameBuffer As Char() = New Char(99) {} ' will never be this long 
            Dim firmwareNameBufferIdx As Integer = 0

            Dim wrongCount As Integer = 0
            Const readState_init As Integer = 0, readState_inFirmwareName_1 As Integer = 1, readState_inFirmwareName_2 As Integer = 2, readState_inFirmwareName_3 As Integer = 3, readState_inFirmwareName_4 As Integer = 4, readState_inFirmwareName_5 As Integer = 5, _
            readState_done As Integer = 6
            Dim readState As Integer = readState_init

            While (readState <> readState_done) AndAlso (wrongCount < 50)
                Dim inputBuffer As Byte() = New Byte(1023) {}
                Dim readLength As UInteger = activePort.ReadPort(inputBuffer, 0, CUInt(inputBuffer.Length))
                If (readLength = 0) Then
                    Exit While
                End If

                For i As Integer = 0 To readLength - 1
                    If (readState = readState_init) AndAlso (inputBuffer(i) = &H1B) Then
                        readState = readState_inFirmwareName_1
                    ElseIf (readState = readState_inFirmwareName_1) AndAlso (inputBuffer(i) = Asc("#"c)) Then
                        readState = readState_inFirmwareName_2
                    ElseIf (readState = readState_inFirmwareName_2) AndAlso (inputBuffer(i) = Asc("*"c)) Then
                        readState = readState_inFirmwareName_3
                    ElseIf (readState = readState_inFirmwareName_3) AndAlso (inputBuffer(i) = Asc(","c)) Then
                        readState = readState_inFirmwareName_4
                    ElseIf (readState = readState_inFirmwareName_4) AndAlso (inputBuffer(i) = &HA) Then
                        readState = readState_inFirmwareName_5
                    ElseIf (readState = readState_inFirmwareName_5) AndAlso (inputBuffer(i) = &H0) Then
                        readState = readState_done
                        Exit For
                    ElseIf readState = readState_inFirmwareName_4 Then
                        ' must be last before the catch-all 
                        firmwareNameBuffer(firmwareNameBufferIdx) = Chr(inputBuffer(i))
                        firmwareNameBufferIdx = firmwareNameBufferIdx + 1
                    Else
                        readState = readState_init
                        wrongCount += 1
                    End If
                Next
            End While

            If readState = readState_done Then
                Console.WriteLine("Success - firmware name = " & New String(firmwareNameBuffer))
            Else
                Console.WriteLine("Failure - could not read in the firmware name.")
            End If
        Catch e As PortException
            Console.WriteLine("Failure - see stack trace:" & vbLf & e.StackTrace)
        Finally
            If activePort IsNot Nothing Then
                StarMicronics.StarIO.Factory.I.ReleasePort(activePort)
            End If
        End Try
    End Sub

    Public Function WritePortHelper(ByVal activePort As IPort, ByVal writeBuffer() As Byte) As UInteger
        Dim zeroProgressOccurnaces As UInteger = 0
        Dim totalSizeCommunicated As UInteger = 0
        While ((totalSizeCommunicated < writeBuffer.Length) And (zeroProgressOccurnaces < 2)) ' adjust zeroProgressOccurances as needed
            Dim sizeCommunicated As UInteger = activePort.WritePort(writeBuffer, totalSizeCommunicated, CUInt(writeBuffer.Length) - totalSizeCommunicated)
            If (sizeCommunicated = 0) Then
                zeroProgressOccurnaces = zeroProgressOccurnaces + 1
            Else
                totalSizeCommunicated = totalSizeCommunicated + sizeCommunicated
                zeroProgressOccurnaces = 0
            End If
        End While

        WritePortHelper = totalSizeCommunicated
    End Function

End Module
